==> nft-sdk-0.0.2.tgz: package/src/modules/fetchNFT.ts <==
import { Connection, PublicKey, Transaction, TransactionInstruction } from "@solana/web3.js";
import { TOKEN_PROGRAM_ID } from '@solana/spl-token'
import { getAssociatedTokenAddress, createAssociatedTokenAccountInstruction, createTransferInstruction } from "spl-token"
import { fetchNFTDetail, getAccountInfo } from "../helper";
import { IFetchNFT } from "../types";


export const fetchNFTByWallet = async (userAddress: string, connection: Connection) => {
  const userTokens = await connection.getParsedTokenAccountsByOwner(new PublicKey(userAddress), {
    programId: TOKEN_PROGRAM_ID
  })

  // filter token has decimals == 0 and token amount !== 0
  const filterTokens = userTokens.value.filter(item => {
    const tokenAmount = item.account.data.parsed?.info?.tokenAmount
    return tokenAmount?.amount !== '0' && tokenAmount?.decimals === 0
  }).map(item => item.account.data.parsed?.info?.mint)

  // fetchAllInfo 
  const nfts = await Promise.all(
    filterTokens.map(async (item) => fetchNFTDetail(item, connection))
  )
  return nfts.filter(item => item !== null)
}

export const buildTransferNFTTransaction = async (fromAddress: string, toAddress: string, mintAddress: string, connection: Connection) => {
  const fromPublicKey = new PublicKey(fromAddress);
  const toPublicKey = new PublicKey(toAddress);

  const fromTokenAccount = await getAssociatedTokenAddress(
    new PublicKey(mintAddress),
    fromPublicKey
  );

  const toTokenAccount = await getAssociatedTokenAddress(
    new PublicKey(mintAddress),
    toPublicKey
  );

  const recipientAccount = await getAccountInfo(toTokenAccount, connection);
  const instructions: TransactionInstruction[] = [];
  if (!recipientAccount || !recipientAccount.isInitialized) {
    instructions.push(
      createAssociatedTokenAccountInstruction(
        fromPublicKey,
        toTokenAccount,
        toPublicKey,
        new PublicKey(mintAddress)
      )
    );
  }
  instructions.push(
    createTransferInstruction(
      fromTokenAccount, // from (should be a token account)
      toTokenAccount,
      fromPublicKey, // from's owner
      1 // amount, if your deciamls is 8, send 10^8 for 1 token
    )
  )
  const transaction = new Transaction().add(...instructions);
  transaction.recentBlockhash = (
    await connection.getLatestBlockhash('finalized')
  ).blockhash;
  transaction.feePayer = fromPublicKey;
  return transaction
}

export class FetchNFT implements IFetchNFT {
  private connection: Connection
  private userAddress: PublicKey
  constructor(connection: Connection, userAddress: PublicKey) {
    this.connection = connection;
    this.userAddress = userAddress
  }

  fetchNFTList = async () => {
    if (!this.connection || !this.userAddress) return [];
    return await fetchNFTByWallet(this.userAddress.toBase58(), this.connection)
  }

  buildTransferTransaction = async (toAddress: string, mintAddress: string) => {
    if (!this.connection || !this.userAddress) return undefined;
    return await buildTransferNFTTransaction(
      this.userAddress.toBase58(),
      toAddress,
      mintAddress,
      this.connection
    )
  }
}

==> nft-sdk-0.0.3.tgz: package/src/modules/fetchNFT.ts <==
import { Connection, PublicKey, Transaction, TransactionInstruction } from "@solana/web3.js";
import { TOKEN_PROGRAM_ID } from '@solana/spl-token'
import { getAssociatedTokenAddress, createAssociatedTokenAccountInstruction, createTransferInstruction } from "spl-token"
import { fetchNFTDetail, getAccountInfo } from "../helper";
import { IFetchNFT, TNFTDetail } from "../types";


export const fetchNFTByWallet = async (userAddress: string, connection: Connection): Promise<TNFTDetail[]> => {
  const userTokens = await connection.getParsedTokenAccountsByOwner(new PublicKey(userAddress), {
    programId: TOKEN_PROGRAM_ID
  })

  // filter token has decimals == 0 and token amount !== 0
  const filterTokens = userTokens.value.filter(item => {
    const tokenAmount = item.account.data.parsed?.info?.tokenAmount
    return tokenAmount?.amount !== '0' && tokenAmount?.decimals === 0
  }).map(item => item.account.data.parsed?.info?.mint)

  // fetchAllInfo 
  const nfts = await Promise.all(
    filterTokens.map(async (item) => fetchNFTDetail(item, connection))
  )
  return nfts.filter(item => item !== null)
}

export const buildTransferNFTTransaction = async (fromAddress: string, toAddress: string, mintAddress: string, connection: Connection) => {
  const fromPublicKey = new PublicKey(fromAddress);
  const toPublicKey = new PublicKey(toAddress);

  const fromTokenAccount = await getAssociatedTokenAddress(
    new PublicKey(mintAddress),
    fromPublicKey
  );

  const toTokenAccount = await getAssociatedTokenAddress(
    new PublicKey(mintAddress),
    toPublicKey
  );

  const recipientAccount = await getAccountInfo(toTokenAccount, connection);
  const instructions: TransactionInstruction[] = [];
  if (!recipientAccount || !recipientAccount.isInitialized) {
    instructions.push(
      createAssociatedTokenAccountInstruction(
        fromPublicKey,
        toTokenAccount,
        toPublicKey,
        new PublicKey(mintAddress)
      )
    );
  }
  instructions.push(
    createTransferInstruction(
      fromTokenAccount, // from (should be a token account)
      toTokenAccount,
      fromPublicKey, // from's owner
      1 // amount, if your deciamls is 8, send 10^8 for 1 token
    )
  )
  const transaction = new Transaction().add(...instructions);
  transaction.recentBlockhash = (
    await connection.getLatestBlockhash('finalized')
  ).blockhash;
  transaction.feePayer = fromPublicKey;
  return transaction
}

export class FetchNFT implements IFetchNFT {
  private connection: Connection
  private userAddress: PublicKey
  constructor(connection: Connection, userAddress: PublicKey) {
    this.connection = connection;
    this.userAddress = userAddress
  }

  fetchNFTList = async () => {
    if (!this.connection || !this.userAddress) return [];
    return await fetchNFTByWallet(this.userAddress.toBase58(), this.connection)
  }

  buildTransferTransaction = async (toAddress: string, mintAddress: string) => {
    if (!this.connection || !this.userAddress) return undefined;
    return await buildTransferNFTTransaction(
      this.userAddress.toBase58(),
      toAddress,
      mintAddress,
      this.connection
    )
  }
}
